# -*- coding: utf-8 -*-

# The MIT/X11/Expat License
# Copyright (c) 2010 Manish Sinha<mail@manishsinha.net>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


class Trophy():
    """ Trophy is the most important entity in OMG

    When dealing with `OMG!` we talk about Trophies,
    trophies are awarded, registsred, deleted. """

    _id = None
    _title = None
    _description = None
    _icon_path = None
    _setname = None
    _application = None
    _app_friendly_name = None
    _priority = 0
    _set_icon = None
    _unlocked = False
    _timestamp = -1
    _stock_icon = None

    def __init__(self, id, title, desc, setname, app, app_friendly_name, \
                     priority, icon_path=None, set_icon=None, \
                    unlocked=False, timestamp=None, stock_icon=None):
        """ Create an instance of the Trophy
        
        :param id: (String) The Unique identifier for each Trophy 
        :param title: (String) The detailed description of the Trophy
        :param desc: (List) The detailed description of the Trophy
        :param setname: (String) The Set under which this trophy belongs to
        :param app: (String) The Application associated with this Trophy
        :param app_friendly_name: (String) The Friendly name of the Application associated with this Trophy
        :param priority: (Integer) The Priority of this Trophy. It should be an integer which can take only three values 0,1,2 otherwise TypeError and ValueError are raised respectivly
        :param icon_path: (String) The Icon Path of the Trophy (Optional)
        :param set_icon: (string) The Set Icon path (Optional)
        :param unlocked: (Boolean) Is the trophy awarded/unlocked (Optional) If supplied boolean values should be passed otherwise TypeError is raised
        :param timestamp: (Integer) The time when the trophy was awarded (Optional) If supplied it should be an integer which can take only positive values otherwise TypeError and ValueError are raised respectivly
        :param stock_icon: (String) The gtk stock icon string for the trophy. eg. gtk-dialog-info. For a full list check http://library.gnome.org/devel/gtk/2.13/gtk-Stock-Items.html If missing then default stock icon can be used (Optional)
        
        """
        
        self._id = str(id)
        self._title = str(title)
        
        if isinstance(desc, list) is False:
            raise TypeError("The description should be a list. The passed argument is %s" %(type(desc).__name__))
        self._desc = desc
        
        self._setname = str(setname)
        self._application = str(app)
        self._app_friendly_name = str(app_friendly_name)
        
        self._priority = int(priority)
        if self._priority not in range(0,3):
            raise ValueError("The range of priority should be 0,1,2. The current value is %d" %(self._priority))
        
        if icon_path is not None:
            self._icon_path = str(icon_path)
        
        if set_icon is not None:
            self._set_icon = str(set_icon)
        
        self._unlocked = bool(unlocked)
        
        if timestamp is not None:
            self._timestamp = int(timestamp)
            if self._timestamp < 0:
                raise ValueError("timestamp is %s which should not be negative" %(timestamp))
        
        if stock_icon is not None:
            self._stock_icon = str(stock_icon)
        
        
    
    @property
    def Id(self):
        """ The Unique identifier for each Trophy
        
        It is usually in the format `Application/SetName/Title`
        
        """
        return self._id

    @property
    def Title(self):
        """ The Title of the Trophy. 
        
        One phrase/line description is enough. Please keep it short
        Example - `Do Addict`
        
        """
        return self._title

    @property
    def Description(self):
        """ The detailed description of the Trophy
        
        The description of the trophy can contain what the user did to 
        achieve this trophy. If the trophy is still unlocked/unawarded then 
        hints can be added on how the trophy can be unlocked
        
        """
        return self._description

    @property
    def IconPath(self):
        """ The Icon Path of the Trophy
        
        The physical location on the disk where the icon can be found. This 
        icon is used for identification of the trophy and also used as the 
        icon of the notification when the trophy is awarded
        
        """
        return self._icon_path

    @property
    def SetName(self):
        """ The Set under which this trophy belongs to 
        
        The set represents the higher level categorization of trophies. 
        Examples of such set can be `Music`, `Development`, `Productivity` etc
        More than one applications can have their trophies in one set and one 
        application can have their trophies in more than one set
        
        """
        return self._setname

    @property
    def Application(self):
        """ The Application associated with this Trophy
        
        The application which had registered the trophy. It has to be of the 
        form `application://appname.desktop`
        
        """
        return self._application

    @property
    def ApplicationFriendlyName(self):
        """ The Friendly name of the Application associated with this Trophy
        
        The Friendly name of the Application which registsred the trophy.
        Example `GNOME Text Editor` can be a friendly name of `gedit`
        
        """
        return self._app_friendly_name

    @property
    def Priority(self):
        """ The Priority of this Trophy 
        
        The Priority is an integral field. It can take only three values - 0,1,2
        Though so such description is attached to these three values, they
        can be considered as `Low`, `Medium` and `High`
        
        """
        return self._priority

    @property
    def SetIcon(self):
        """ The Set Icon path
        
        It is now of use. To get the icon of the set, please check 
        the folder `~/.local/share/omg/seticon/` with same name as set. 
        If the corrosponding icon is not present, then a random stock icon 
        can be used
        
        """
        return self._set_icon

    @property
    def Unlocked(self):
        """ Is the trophy awarded/unlocked
        
        If the value is set as True, then the trophy is unlocked or awarded
        
        """
        return self._unlocked

    @property
    def Timestamp(self):
        """ The time when the trophy was awarded
        
        The time when the trophy was awarded/unlocked. It is valid only when 
        Unlocked Property is True
        
        """
        return self._timestamp

    def StockIcon(self):
        """ The stock icon for the trophy. If missing then default can be used
        
        The stock icon used when IconPath is None or when no such icon 
        exists on the disk. This is a fallback for `IconPath`
        eg. `gtk-dialog-info`. For a full list of stock icons 
        check http://library.gnome.org/devel/gtk/2.13/gtk-Stock-Items.html
        
        """
        return self._stock_icon